@@ -5,8 +5,9 @@ asvar <- function(object, name, ...)
 asvar.factor <- function(object, name, ...)  
     variables::factor_var(name, levels = levels(object))
 
-asvar.ordered <- function(object, name, ...)  
-    variables::ordered_var(name, levels = levels(object))
+asvar.ordered <- function(object, name, sparse_nlevels = Inf, ...)
+    variables::ordered_var(name, levels = levels(object),
+                           sparse = nlevels(object) >= sparse_nlevels)
 
 asvar.numeric <- function(object, name, prob = c(.1, .9), support = NULL, bounds = NULL, add = c(0, 0), ...) {
     if (is.integer(object)) {
@@ -25,9 +26,10 @@ asvar.numeric <- function(object, name, prob = c(.1, .9), support = NULL, bounds
 asvar.Surv <- function(object, name, prob = c(.1, .9), support = NULL, bounds = c(0, Inf), add = c(0, 0), ...) {
     if (is.null(bounds)) bounds <- c(0, Inf)
     if (is.null(support)) {
-        support <- quantile(survfit(y ~ 1, data = data.frame(y = object)), prob = prob)$quantile
+        support <- quantile(sf <- survfit(y ~ 1, data = data.frame(y = object)), prob = prob)$quantile
+        last <- max(sf$time[sf$n.event > 0])
         if (is.na(support[2])) 
-            support[2] <- max(object[, 1])
+            support[2] <- last
         add <- c(0 - support[1], 0)
         return(variables::numeric_var(name, support = support, bounds = bounds, add = add, ...))
     }
@@ -37,14 +39,15 @@ asvar.Surv <- function(object, name, prob = c(.1, .9), support = NULL, bounds =
 asvar.response <- function(object, name, prob = c(.1, .9), support = NULL, bounds = c(-Inf, Inf), ...) {
     if (any(sapply(object, is.factor))) {
         f <- object$exact
-        if (any(!is.na(f))) return(asvar(f, name))
-        if (any(!is.na(object$cleft))) return(asvar(object$cleft, name))
+        if (any(!is.na(f))) return(asvar(f, name, ...))
+        if (any(!is.na(object$cleft))) return(asvar(object$cleft, name, ...))
         stop("cannot determine class of response")
     }
     if (is.null(support)) {
-        support <- quantile(survfit(y ~ 1, data = data.frame(y = as.Surv(object))), prob = prob)$quantile
+        support <- quantile(sf <- survfit(y ~ 1, data = data.frame(y = as.Surv(object))), prob = prob)$quantile
+        last <- max(sf$time[sf$n.event > 0])
         if (is.na(support[2])) 
-            support[2] <- max(object$approxy)
+            support[2] <- last
     }
     variables::numeric_var(name, support = support, bounds = bounds, ...)
 }
