@@ -2,13 +2,14 @@
 ### marginally interpretable linear transformation models for clustered
 ### observations
 
-mtram <- function(object, formula, data, standardise = FALSE,
+mtram <- function(object, formula, data, 
                   grd = SparseGrid::createSparseGrid(type = "KPU", dimension = length(rt$cnms[[1]]), 
                                                      k = 10),
                   Hessian = FALSE, tol = .Machine$double.eps,
-                  grad_num = FALSE,
+                  # standardise = FALSE,
                   ...) {
     
+    standardise <- TRUE
     stopifnot(inherits(object, "mlt_fit"))
     
     bar.f <- lme4::findbars(formula)
@@ -45,19 +46,15 @@ mtram <- function(object, formula, data, standardise = FALSE,
         attr(iY$Yleft, "constraint") <- tmp
         attr(iY$Yright, "constraint") <- tmp
     }
-    if (length(eY$which) > 0 && length(iY$which))
+    if ((length(eY$which) > 0) && (length(iY$which) > 0))
         stop("cannot deal with mixed censoring")
     
-    if (length(iY$which) > 0 && grad_num)
-        stop("grad_num argument only available for continuous responses")
-    
     w <- object$weights
     
     NORMAL <- FALSE
     if (object$todistr$name == "normal") {
         NORMAL <- TRUE
         PF <- function(z) z
-        if(standardise) message("standardise = TRUE argument is ignored")
     } else {
         P <- object$todistr$p
         PF <- function(z) qnorm(pmin(1 - tol, pmax(tol, P(z))))
@@ -68,13 +65,8 @@ mtram <- function(object, formula, data, standardise = FALSE,
     gr <- NULL
     
     ### catch constraint violations here
-    .log <- function(x) {
-        return(log(pmax(.Machine$double.eps, x)))
-        pos <- (x > .Machine$double.eps)
-        if (all(pos)) return(log(x))
-        ret[pos] <- log(x[pos])
-        return(ret)
-    }
+    .log <- function(x) 
+        log(pmax(.Machine$double.eps, x))
     
     ## continuous case
     if (length(eY$which) > 0) {
@@ -105,7 +97,8 @@ mtram <- function(object, formula, data, standardise = FALSE,
             DPFz <- c(D * PFz)
             
             if (NORMAL) {
-                ret <- -0.5 * (logdet + sum((Linv %*% DPFz)^2) - sum(DPFz^2)) - object$loglik(theta, weights = w)
+                ret <- -0.5 * (logdet + sum((Linv %*% DPFz)^2) - sum(DPFz^2)) -
+                    object$loglik(theta, weights = w)
             } else {
                 ret <- -0.5 * (logdet + sum((Linv %*% DPFz)^2) - sum(PFz^2)) +
                     sum(.log(f(Dinvz) * c(eY$Yprime %*% theta)))
@@ -115,7 +108,7 @@ mtram <- function(object, formula, data, standardise = FALSE,
         
         X <- eY$Y
         
-        if(NORMAL) { ## M1 or M2, probit link
+        if(NORMAL) { ## probit link
             gr <- function(parm) {
                 theta <- parm[1:ncol(eY$Y)]
                 gamma <- parm[-(1:ncol(eY$Y))]
@@ -146,101 +139,64 @@ mtram <- function(object, formula, data, standardise = FALSE,
                 return(-c(c(as(dtheta, "matrix")), dgamma))
             }
         } else { ## no probit link
-            if(!standardise) { ## M1, no probit link
-                gr <- function(parm) {
-                    theta <- parm[1:ncol(eY$Y)]
-                    gamma <- parm[-(1:ncol(eY$Y))]
-                    devLambda <- devSigma <- vector(mode = "list",
-                                                    length = length(gamma))
-                    dgamma <- numeric(length(gamma))
-                    z <- c(eY$Y %*% theta + offset)
-                    PFz <- c(PF(z))
-                    
-                    Lambdat@x[] <- mapping(gamma)
-                    L <- update(L, t(Lambdat %*% ZtW), mult = 1)
-                    LM <- as(L, "Matrix")
-                    Linv <- solve(LM)
-                    # Sigma <- tcrossprod(LM)
-                    SigmaInv <- crossprod(Linv)
-                    LambdaInd <- t(Lambdat)
-                    LambdaInd@x[] <- 1:length(gamma)
-                    for (i in 1:length(gamma)) {
-                        ### Wang & Merkle (2018, JSS) compute derivative of G!
-                        ### We need derivative of Lambda!
-                        dLtL <- (LambdaInd == i) %*% Lambdat
-                        devLambda[[i]] <- dLtL + t(dLtL)
-                        devSigma[[i]] <- crossprod(ZtW, devLambda[[i]] %*% ZtW)
-                        t1 <- SigmaInv %*% devSigma[[i]]
-                        t2 <- t1 %*% SigmaInv
-                        dgamma[i] <- -.5 * (sum(diag(t1)) - crossprod(PFz, t2) %*% PFz)
-                    }
-                    Si1 <- SigmaInv
-                    diag(Si1) <- diag(Si1) - 1
-                    dtheta <- - crossprod(PFz, Si1) %*%
-                        ((f(z)/dnorm(PFz)) * eY$Y) +
-                        colSums((fprime(z)/f(z)) %*% eY$Y) +
-                        colSums(eY$Yprime / c(eY$Yprime %*% theta))
-                    return(-c(c(as(dtheta, "matrix")), dgamma))
-                }
-            } else { ## M2, no probit link
-                gr <- function(parm) {
-                    theta <- parm[1:ncol(eY$Y)]
-                    gamma <- parm[-(1:ncol(eY$Y))]
-                    devLambda <- devSigma <- vector(mode = "list",
-                                                    length = length(gamma))
-                    
-                    Lambdat@x[] <- mapping(gamma)
-                    L <- update(L, t(Lambdat %*% ZtW), mult = 1)
-                    LM <- as(L, "Matrix")
-                    Linv <- solve(LM)
-                    # Sigma <- tcrossprod(LM)
-                    SigmaInv <- crossprod(Linv)
-                    D2 <- rowSums(LM^2) # diag(Sigma)
-                    D <- sqrt(D2)
-                    Dinv <- 1/D
-                    Dinv2 <- 1/D2
-                    LambdaInd <- t(Lambdat)
-                    LambdaInd@x[] <- 1:length(gamma)
-                    
-                    dgamma <- numeric(length(gamma))
-                    z <-  c(eY$Y %*% theta + offset)
-                    Dinvz <- Dinv * z
-                    PFz <- c(PF(Dinvz))
-                    DPFz <- c(D * PFz)
-                    
-                    for (i in 1:length(gamma)) {
-                        ### Wang & Merkle (2018, JSS) compute derivative of G!
-                        ### We need derivative of Lambda!
-                        dLtL <- (LambdaInd == i) %*% Lambdat
-                        devLambda[[i]] <- dLtL + t(dLtL)
-                        devSigma[[i]] <- crossprod(ZtW, devLambda[[i]] %*% ZtW)
-                        t1 <- SigmaInv %*% devSigma[[i]]
-                        t2 <- t1 %*% SigmaInv
-                        dDPFz <- .5 * Dinv2 * diag(devSigma[[i]]) *
-                            (DPFz - (c(f(Dinvz)) / dnorm(PFz)) * z)
-                        dDinv <- -.5 * D^(-3) * diag(devSigma[[i]])
-                        dDinv2 <- - D2^(-2) * diag(devSigma[[i]])
-                        t3 <- t2
-                        diag(t3) <- diag(t3) + dDinv2
-                        SiID2 <- SigmaInv
-                        diag(SiID2) <- diag(SiID2) - Dinv2
-                        dgamma[i] <- -.5 * (sum(diag(t1)) +
-                                                crossprod(dDPFz, SiID2) %*% DPFz -
-                                                crossprod(DPFz, t3) %*% DPFz +
-                                                crossprod(DPFz, SiID2) %*% dDPFz) +
-                            sum(c(fprime(Dinvz))/c(f(Dinvz)) * c(dDinv * z))
-                    }
-                    dtheta <- - crossprod(DPFz, SiID2) %*%
-                        ((c(f(Dinvz))/dnorm(PFz)) * eY$Y) +
-                        colSums((c(fprime(Dinvz))/c(f(Dinvz))) * (Dinv * eY$Y)) +
-                        colSums(eY$Yprime / c(eY$Yprime %*% theta))
-                    
-                    return(-c(c(as(dtheta, "matrix")), dgamma))
+            gr <- function(parm) {
+                theta <- parm[1:ncol(eY$Y)]
+                gamma <- parm[-(1:ncol(eY$Y))]
+                devLambda <- devSigma <- vector(mode = "list",
+                                                length = length(gamma))
+                
+                Lambdat@x[] <- mapping(gamma)
+                L <- update(L, t(Lambdat %*% ZtW), mult = 1)
+                LM <- as(L, "Matrix")
+                Linv <- solve(LM)
+                # Sigma <- tcrossprod(LM)
+                SigmaInv <- crossprod(Linv)
+                D2 <- rowSums(LM^2) # diag(Sigma)
+                D <- sqrt(D2)
+                Dinv <- 1/D
+                Dinv2 <- 1/D2
+                LambdaInd <- t(Lambdat)
+                LambdaInd@x[] <- 1:length(gamma)
+                
+                dgamma <- numeric(length(gamma))
+                z <-  c(eY$Y %*% theta + offset)
+                Dinvz <- Dinv * z
+                PFz <- c(PF(Dinvz))
+                DPFz <- c(D * PFz)
+                
+                for (i in 1:length(gamma)) {
+                    ### Wang & Merkle (2018, JSS) compute derivative of G!
+                    ### We need derivative of Lambda!
+                    dLtL <- (LambdaInd == i) %*% Lambdat
+                    devLambda[[i]] <- dLtL + t(dLtL)
+                    devSigma[[i]] <- crossprod(ZtW, devLambda[[i]] %*% ZtW)
+                    t1 <- SigmaInv %*% devSigma[[i]]
+                    t2 <- t1 %*% SigmaInv
+                    dDPFz <- .5 * Dinv2 * diag(devSigma[[i]]) *
+                        (DPFz - (c(f(Dinvz)) / dnorm(PFz)) * z)
+                    dDinv <- -.5 * D^(-3) * diag(devSigma[[i]])
+                    dDinv2 <- - D2^(-2) * diag(devSigma[[i]])
+                    t3 <- t2
+                    diag(t3) <- diag(t3) + dDinv2
+                    SiID2 <- SigmaInv
+                    diag(SiID2) <- diag(SiID2) - Dinv2
+                    dgamma[i] <- -.5 * (sum(diag(t1)) +
+                                            crossprod(dDPFz, SiID2) %*% DPFz -
+                                            crossprod(DPFz, t3) %*% DPFz +
+                                            crossprod(DPFz, SiID2) %*% dDPFz) +
+                        sum(c(fprime(Dinvz))/c(f(Dinvz)) * c(dDinv * z))
                 }
+                dtheta <- - crossprod(DPFz, SiID2) %*%
+                    ((c(f(Dinvz))/dnorm(PFz)) * eY$Y) +
+                    colSums((c(fprime(Dinvz))/c(f(Dinvz))) * (Dinv * eY$Y)) +
+                    colSums(eY$Yprime / c(eY$Yprime %*% theta))
+                
+                return(-c(c(as(dtheta, "matrix")), dgamma))
             }
         }
     } else { ## censored and discrete case
-        stopifnot(length(rt$flist) == 1)
+        if (length(rt$flist) != 1L)
+            stop("only one grouping factor allowed for discrete case")
         grp <- rt$flist[[1]]
         idx <- split(1:length(grp), grp)
         wh <- 1:length(rt$cnms[[1]])
@@ -281,7 +237,7 @@ mtram <- function(object, formula, data, standardise = FALSE,
                 .Marsaglia_1963(zlower, zupper, mean = 0, V = V, 
                                 do_qnorm = FALSE, grd = grd)
             })
-            return(-sum(log(pmax(.Machine$double.eps, ret))))
+            return(-sum(.log(ret)))
         }
         X <- iY$Yleft
     }            
@@ -293,8 +249,6 @@ mtram <- function(object, formula, data, standardise = FALSE,
     
     start <- c(coef(as.mlt(object), fixed = FALSE), theta)
     
-    if(grad_num) gr <- NULL
-    
     if (is.null(gr)) {
         opt <- alabama::auglag(par = start, fn = ll, 
                                hin = function(par) ui %*% par - ci, 
@@ -306,13 +260,9 @@ mtram <- function(object, formula, data, standardise = FALSE,
                                hin.jac = function(par) ui,
                                control.outer = list(trace = FALSE))[c("par", "value", "gradient")]
     }
-    if (length(eY$which) > 0) {
-        gamma <- opt$par[-(1:ncol(eY$Y))]
-        names(opt$par)[-(1:ncol(eY$Y))] <- paste0("gamma", 1:length(gamma))
-    } else {
-        gamma <- opt$par[-(1:ncol(iY$Yleft))]
-        names(opt$par)[-(1:ncol(iY$Yleft))] <- paste0("gamma", 1:length(gamma))
-    }
+
+    gamma <- opt$par[-(1:ncol(X))]
+    names(opt$par)[-(1:ncol(X))] <- paste0("gamma", 1:length(gamma))
     Lambdat@x[] <- mapping(gamma)
     opt$G <- crossprod(Lambdat)[1:length(rt$cnms[[1]]),1:length(rt$cnms[[1]])]
     if (Hessian) opt$Hessian <- numDeriv::hessian(ll, opt$par)
